/*
   FNT - Daemon for handle xmpp-connections
   Reading application settings

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <gnutls/gnutls.h>
#include <gnutls/x509.h>

#include "../include/global.h"
#include "../include/types.h"
#include "../include/sql.h"
#include "../include/sql-config.h"
#include "../include/log.h"
#include "../include/ssltls.h"

#include "settings.h"
#include "main.h"

/*** global variables **************************************************/

xmppd_settings_t *settings = NULL;

/*** file scope macro definitions **************************************/

#define CONFIG_GROUP PACKAGE"-xmppd"

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

void
read_one_setting_str (char **dest, const char *param,
		      const char *default_value)
{
    *dest = fnt_sql_config_getParameter (config_connect, CONFIG_GROUP, param);
    if (*dest == NULL)
	*dest = g_strdup (default_value);
}

void
read_one_setting_int (int *dest, const char *param, int default_value)
{
    char *tmp_param;

    tmp_param =
	fnt_sql_config_getParameter (config_connect, CONFIG_GROUP, param);

    if (tmp_param != NULL)
    {
	*dest = atoi (tmp_param);
	g_free (tmp_param);
	return;
    }
    *dest = default_value;
}

void
read_one_setting_bool (bool * dest, const char *param, bool default_value)
{
    char *tmp_param;

    tmp_param =
	fnt_sql_config_getParameter (config_connect, CONFIG_GROUP, param);

    if (tmp_param != NULL)
    {
	if (!g_ascii_strcasecmp (tmp_param, "true")
	    || !g_ascii_strcasecmp (tmp_param, "yes")
	    || !g_ascii_strcasecmp (tmp_param, "1"))
	{
	    *dest = true;
	}
	else
	{
	    *dest = false;
	}
	g_free (tmp_param);
	return;
    }
    *dest = default_value;
}

void
read_common_settings (void)
{
    char *tmp_param;

    settings->pid_directory =
	fnt_sql_config_getParameter (config_connect, "common-settings",
				     "pid.directory");

    if (settings->pid_directory == NULL)
	settings->pid_directory = g_strdup (".");

    settings->ssl_directory =
	fnt_sql_config_getParameter (config_connect, "common-settings",
				     "ssl.directory");
    if (settings->ssl_directory == NULL)
	settings->ssl_directory = g_strdup (".");

    settings->fidonet_node_address =
	fnt_sql_config_getParameter (config_connect, "fidonet",
				     "node.address");
    if (settings->fidonet_node_address == NULL)
	settings->fidonet_node_address = g_strdup ("");


    tmp_param =
	fnt_sql_config_getParameter (config_connect, CONFIG_GROUP,
				     "log.type");
    if (tmp_param == NULL)
    {
	settings->log_type = FNT_LOG_T_NONE;
    }
    else
    {
	settings->log_type =
	    fnt_log_getLogTypeByName ((const char *) tmp_param);
	g_free (tmp_param);
    }

    read_one_setting_int (&(settings->log_level), "log.level", LOG_critical);


    switch (settings->log_type)
    {
    case FNT_LOG_T_SYSLOG:
	read_one_setting_str (&(settings->facility_or_fname), "log.facility",
			      "local7");

	break;
    case FNT_LOG_T_FILE:
	read_one_setting_str (&(settings->facility_or_fname), "log.file",
			      CONFIG_GROUP ".log");
	break;
    default:
	settings->facility_or_fname = NULL;
	break;
    }

}

/*** public functions **************************************************/

void
cleanup_settings (void)
{
    if (settings == NULL)
	return;

    g_free (settings->listen_ip);
    g_free (settings->pid_file);
    g_free (settings->pid_directory);
    g_free (settings->ssl_directory);
    g_free (settings->ssl_private_key);
    g_free (settings->ssl_public_key);

    g_free (settings->fidonet_node_address);
    if (settings->facility_or_fname != NULL)
	g_free (settings->facility_or_fname);

    g_free (settings);
    settings = NULL;
}

bool
read_settings (void)
{
    char *tmp_param;

    if (settings != NULL)
	cleanup_settings ();

    settings = g_try_malloc (sizeof (xmppd_settings_t));
    if (settings == NULL)
	return false;

    read_common_settings ();

    read_one_setting_str (&(settings->pid_file), "pid.file",
			  CONFIG_GROUP ".pid");

    read_one_setting_int (&(settings->io_max_fds), "io.max_fds", 1024);
    read_one_setting_int (&(settings->io_check_interval), "io.check_interval",
			  0);


    read_one_setting_str (&(settings->listen_ip), "listen.ip", "0.0.0.0");

    read_one_setting_int (&(settings->listen_port), "listen.port", 24444);


    read_one_setting_str (&tmp_param, "ssl.auth.type", "anonymous");
//    settings->tls_auth_type = (int) fnt_ssltls_auth_type_getType (tmp_param);
    g_free (tmp_param);

    read_one_setting_bool (&(settings->debug_socket), "debug.socket", false);
    read_one_setting_bool (&(settings->debug_stream), "debug.stream", false);
    read_one_setting_bool (&(settings->debug_session), "debug.session",
			   false);

    read_one_setting_str (&(settings->ssl_private_key), "ssl.key.private", "");
    read_one_setting_str (&(settings->ssl_public_key), "ssl.key.public", "");

    return true;
}
